@@ -4,11 +4,11 @@ use strict;
 use warnings;
 use autodie;
 use feature qw(say);
-use Data::Dumper;
-use Convert::Pheno::Default qw(get_defaults);
+use Sys::Hostname;
+use Cwd qw(cwd abs_path);
 use Convert::Pheno::Mapping;
 use Exporter 'import';
-our @EXPORT = qw(do_pxf2bff);
+our @EXPORT = qw(do_pxf2bff get_metaData);
 
 #############
 #############
@@ -16,38 +16,32 @@ our @EXPORT = qw(do_pxf2bff);
 #############
 #############
 
-my $DEFAULT = get_defaults();
-
 sub do_pxf2bff {
 
     my ( $self, $data ) = @_;
     my $sth = $self->{sth};
 
-    # *** IMPORTANT ****
-    # PXF three top-level elements are usually split in files:
-    # - phenopacket.json ( usually - 1 individual per file)
-    # - cohort.json (info on mutliple individuals)
-    # - family.json (info related to one or multiple individuals).
-    # These 3 files dont't contain their respective objects at the root level (/).
-    #
-    # However, top-elements might be combined into a single file (e.g., pxf.json),
-    # as a result, certain files may contain objects for top-level elements:
-    # - /phenopacket
-    # - /cohort
-    # - /family
-    #
-    # In this context, we only accept top-level phenopackets,
-    # while the other two types will be categorized as "info".
+  # *** IMPORTANT ****
+  # PXF three top-level elements are usually split in files:
+  # - phenopacket.json ( usually - 1 individual per file)
+  # - cohort.json (info on mutliple individuals)
+  # - family.json (info related to one or multiple individuals).
+  # These 3 files dont't contain their respective objects at the root level (/).
+  #
+  # However, top-elements might be combined into a single file (e.g., pxf.json),
+  # as a result, certain files may contain objects for top-level elements:
+  # - /phenopacket
+  # - /cohort
+  # - /family
+  #
+  # In this context, we only accept top-level phenopackets,
+  # while the other two types will be categorized as "info".
 
     # We create cursors for top-level elements
     # 1 - phenopacket (mandatory)
     my $phenopacket =
       exists $data->{phenopacket} ? $data->{phenopacket} : $data;
 
-    # Validate format
-    die "Are you sure that your input is not already a bff?\n"
-      unless validate_format( $phenopacket, 'pxf' );
-
     # 2, 3 - /cohort and /family (unlikely)
     # NB: They usually contain info on many individuals and their own files)
     my $cohort = exists $data->{family} ? $data->{cohort} : undef;
@@ -56,18 +50,30 @@ sub do_pxf2bff {
     # Normalize the hash for medical_actions + medicalActions = medicalActions
     if ( exists $phenopacket->{medical_actions} ) {
 
-        # NB: The delete function returns the value of the deleted key-value pair
+       # NB: The delete function returns the value of the deleted key-value pair
         $phenopacket->{medicalActions} = delete $phenopacket->{medical_actions};
     }
 
-    # CNAG files have 'meta_data' nomenclature, but PXF documentation uses 'metaData'
-    # We search for both 'meta_data' and 'metaData' and simply display the
+# CNAG files have 'meta_data' nomenclature, but PXF documentation uses 'metaData'
+# We search for both 'meta_data' and 'metaData' and simply display the
     if ( exists $phenopacket->{meta_data} ) {
 
-        # NB: The delete function returns the value of the deleted key-value pair
+       # NB: The delete function returns the value of the deleted key-value pair
         $phenopacket->{metaData} = delete $phenopacket->{meta_data};
     }
 
+    # Define defaults
+    my $default_date            = '1900-01-01';
+    my $default_duration        = 'P999Y';
+    my $default_value           = -1;
+    my $default_timestamp       = '1900-01-01T00:00:00Z';
+    my $default_iso8601duration = { iso8601duration => $default_duration };
+    my $default_ontology        = { id => 'NCIT:NA0000', label => 'NA' };
+    my $default_quantity        = {
+        unit  => $default_ontology,
+        value => $default_value
+    };
+
     ####################################
     # START MAPPING TO BEACON V2 TERMS #
     ####################################
@@ -93,11 +99,10 @@ sub do_pxf2bff {
             $disease->{diseaseCode} = $disease->{term};
             $disease->{ageOfOnset}  = $disease->{onset}
               if exists $disease->{onset};
-
-            # Check and normalize keys if they exist
-            for (qw/excluded negated/) {
-                $disease->{$_} = $disease->{$_} if exists $disease->{$_};
-            }
+            $disease->{excluded} =
+              ( exists $disease->{negated} || exists $disease->{excluded} )
+              ? JSON::XS::true
+              : JSON::XS::false;
 
             # Clean analog terms if exist
             for (qw/term onset/) {
@@ -125,10 +130,10 @@ sub do_pxf2bff {
               substr( $exposure->{occurrence}{timestamp}, 0, 10 );
 
             # Required properties
-            $exposure->{ageAtExposure} = $DEFAULT->{iso8601duration};
-            $exposure->{duration}      = $DEFAULT->{duration};
+            $exposure->{ageAtExposure} = $default_iso8601duration;
+            $exposure->{duration}      = $default_duration;
             unless ( exists $exposure->{unit} ) {
-                $exposure->{unit} = $DEFAULT->{ontology_term};
+                $exposure->{unit} = $default_ontology;
             }
 
             # Clean analog terms if exist
@@ -184,11 +189,11 @@ sub do_pxf2bff {
                 $procedure->{procedureCode} =
                   exists $action->{procedure}{code}
                   ? $action->{procedure}{code}
-                  : $DEFAULT->{ontology_term};
+                  : $default_ontology;
                 $procedure->{ageOfProcedure} =
                   exists $action->{procedure}{performed}
                   ? $action->{procedure}{performed}
-                  : $DEFAULT->{timestamp};
+                  : $default_timestamp;
 
                 # Clean analog terms if exist
                 for (qw/code performed/) {
@@ -226,7 +231,7 @@ sub do_pxf2bff {
             $measure->{measurementValue} =
                 exists $measure->{value}        ? $measure->{value}
               : exists $measure->{complexValue} ? $measure->{complexValue}
-              :                                   $DEFAULT->{value};
+              :                                   $default_value;
             $measure->{observationMoment} = $measure->{timeObserved}
               if exists $measure->{timeObserved};
 
@@ -255,13 +260,12 @@ sub do_pxf2bff {
             # *** IMPORTANT ****
             # In v2.0.0 BFF 'evidence' is object but in PXF is array of objects
 
-            # Check and normalize keys if they exist
-            for (qw/excluded negated/) {
-                $phenotypicFeature->{excluded} = $phenotypicFeature->{$_}
-                  if exists $phenotypicFeature->{$_};
-            }
-
-            $phenotypicFeature->{featureType} = $phenotypicFeature->{type}
+            $phenotypicFeature->{excluded} =
+              (      exists $phenotypicFeature->{negated}
+                  || exists $phenotypicFeature->{excluded} )
+              ? JSON::XS::true
+              : JSON::XS::false,
+              $phenotypicFeature->{featureType} = $phenotypicFeature->{type}
               if exists $phenotypicFeature->{type};
 
             # Clean analog terms if exist
@@ -278,7 +282,7 @@ sub do_pxf2bff {
     # sex
     # ===
 
-    $individual->{sex} = map_ontology_term(
+    $individual->{sex} = map_ontology(
         {
             query    => $phenopacket->{subject}{sex},
             column   => 'label',
@@ -300,7 +304,7 @@ sub do_pxf2bff {
                 $treatment->{treatmentCode} =
                   exists $action->{treatment}{agent}
                   ? $action->{treatment}{agent}
-                  : $DEFAULT->{ontology_term};
+                  : $default_ontology;
 
                 # Clean analog terms if exist
                 delete $treatment->{agent}
@@ -317,12 +321,12 @@ sub do_pxf2bff {
 
                         # quantity
                         unless ( exists $_->{quantity} ) {
-                            $_->{quantity} = $DEFAULT->{quantity};
+                            $_->{quantity} = $default_quantity;
                         }
 
                         #scheduleFrequency
                         unless ( exists $_->{scheduleFrequency} ) {
-                            $_->{scheduleFrequency} = $DEFAULT->{ontology_term};
+                            $_->{scheduleFrequency} = $default_ontology;
                         }
                     }
                 }
@@ -368,13 +372,84 @@ sub map_complexValue {
     return 1;
 }
 
-# Function to normalize a value to a Boolean
-sub to_boolean {
+sub get_metaData {
 
-    my $value = shift;
-    print Dumper $value;
-    return JSON::XS::true if $value && $value ne 'false';    # Non-empty string and not 'false'
-    return JSON::XS::false;                                  # Empty, 'false', or undef
+    my $self = shift;
+
+    # NB: Q: Why inside PXF.pm and not inside BFF.pm?
+    #   : A: Because it's easier to remember (used in REDCap,pm, BFF.pm)
+
+    # Setting a few variables
+    my $user = $self->{username};
+
+   # NB: Darwin does not have nproc to show #logical-cores, using sysctl instead
+    my $os = $^O;
+    chomp(
+        my $ncpuhost =
+          lc($os) eq 'darwin' ? qx{/usr/sbin/sysctl -n hw.logicalcpu}
+        : $os eq 'MSWin32' ? qx{wmic cpu get NumberOfLogicalProcessors}
+        :                    qx{/usr/bin/nproc} // 1
+    );
+
+    # For the Windows command, the result will also contain the string
+    # "NumberOfLogicalProcessors" which is the header of the output.
+    # So we need to extract the actual number from it:
+    if ( $os eq 'MSWin32' ) {
+        ($ncpuhost) = $ncpuhost =~ /(\d+)/;
+    }
+    $ncpuhost = 0 + $ncpuhost;    # coercing it to be a number
+
+    my $info = {
+        user            => $user,
+        ncpuhost        => $ncpuhost,
+        cwd             => cwd,
+        hostname        => hostname,
+        'Convert-Pheno' => $::VERSION
+    };
+    my $resources = [
+        {
+            id   => 'ICD10',
+            name =>
+'International Statistical Classification of Diseases and Related Health Problems 10th Revision',
+            url             => 'https://icd.who.int/browse10/2019/en#',
+            version         => '2019',
+            namespacePrefix => 'ICD10',
+            iriPrefix       => 'https://icd.who.int/browse10/2019/en#/'
+        },
+        {
+            id              => 'NCIT',
+            name            => 'NCI Thesaurus',
+            url             => 'http://purl.obolibrary.org/obo/ncit.owl',
+            version         => '22.03d',
+            namespacePrefix => 'NCIT',
+            iriPrefix       => 'http://purl.obolibrary.org/obo/NCIT_'
+        },
+        {
+            id              => 'Athena-OHDSI',
+            name            => 'Athena-OHDSI',
+            url             => 'https://athena.ohdsi.org',
+            version         => 'v5.3.1',
+            namespacePrefix => 'OHDSI',
+            iriPrefix       => 'http://www.fakeurl.com/OHDSI_'
+        }
+    ];
+    return {
+        #_info => $info,         # Not allowed
+        created                  => iso8601_time(),
+        createdBy                => $user,
+        submittedBy              => $user,
+        phenopacketSchemaVersion => '2.0',
+        resources                => $resources,
+        externalReferences       => [
+            {
+                id        => 'PMID: 26262116',
+                reference =>
+                  'https://www.ncbi.nlm.nih.gov/pmc/articles/PMC4815923',
+                description =>
+'Observational Health Data Sciences and Informatics (OHDSI): Opportunities for Observational Researchers'
+            }
+        ]
+    };
 }
 
 1;
